@@ -19,7 +19,7 @@ from ..config import Config
 from ..verification import Verifier
 from ..auth import get_authenticator, Action
 from ..errors import StorageError, AuthenticationError
-from ..util import call_from_path, catchtime
+from ..util import call_from_path, catchtime, Periodic
 from . import get_metadata
 
 
@@ -106,7 +106,7 @@ class DBStorage(BaseStorage):
                 await conn.execute(sa.text("PRAGMA optimize"))
 
     async def setup(self):
-        self.log.info("Database URL: '%s'", self.db_url)
+        # self.log.info("Database URL: '%s'", self.db_url)
         from sqlalchemy.ext.asyncio import create_async_engine
 
         self.authenticator = get_authenticator(self, Config.get('authentication', {}))
@@ -305,6 +305,9 @@ class DBStorage(BaseStorage):
         self.log.debug('%s/%s filters: %s', client_id, sub_id, filters)
         if sub_id in self.clients[client_id]:
             await self.unsubscribe(client_id, sub_id)
+
+        if Config.subscription_limit and len(self.clients[client_id]) == Config.subscription_limit:
+            raise StorageError("rejected: too many subscriptions")
         sub = Subscription(
                 self.db,
                 sub_id,
@@ -397,7 +400,7 @@ class DBStorage(BaseStorage):
             'names': {},
             'relays': {}
         }
-        self.log.debug("Getting identity for ? ?", identifier, domain)
+        self.log.debug("Getting identity for %s %s", identifier, domain)
         async with self.db.begin() as conn:
             result = await conn.stream(query)
             async for pubkey, identifier, relays in result:
@@ -521,7 +524,7 @@ class Subscription:
                 if not isinstance(value, list):
                     value = [value]
                 ids = set(value)
-                if ids:
+                if ids and len(ids) < 40: # pathological queries?
                     exact = []
                     for eid in ids:
                         eid = validate_id(eid)
@@ -535,7 +538,7 @@ class Subscription:
                                 if self.is_postgres:
                                     subwhere.append(f"encode(id, 'hex') LIKE '{eid}%'")
                                 else:
-                                    subwhere.append(f"lower(hex(id) LIKE '{eid}%')")
+                                    subwhere.append(f"lower(hex(id)) LIKE '{eid}%'")
                     if exact:
                         idstr = ','.join(exact)
                         subwhere.append(f'events.id IN ({idstr})')
@@ -543,13 +546,22 @@ class Subscription:
                     # invalid query
                     raise ValueError("ids")
             elif key == 'authors' and isinstance(value, list):
-                if value:
-                    if self.is_postgres:
-                        astr = ','.join("'\\x%s'" % validate_id(a) for a in set(value))
-                    else:
-                        astr = ','.join("x'%s'" % validate_id(a) for a in set(value))
-                    if astr:
-                        subwhere.append(f"(pubkey IN ({astr}) OR id IN (SELECT id FROM tags WHERE name = 'delegation' AND value IN ({astr})))")
+                if value and len(value) < 40:
+                    exact = set()
+                    hexexact = set()
+                    for pubkey in value:
+                        pubkey = validate_id(pubkey)
+                        if pubkey:
+                            if len(pubkey) == 64:
+                                if self.is_postgres:
+                                    exact.add(f"'\\x{pubkey}'")
+                                else:
+                                    exact.add(f"x'{pubkey}'")
+                                hexexact.add(f"'{pubkey}'")
+                            # no prefix searches, for now
+                    if exact:
+                        astr = ','.join(exact)
+                        subwhere.append(f"(pubkey IN ({astr}) OR id IN (SELECT id FROM tags WHERE name = 'delegation' AND value IN ({','.join(hexexact)})))")
                     else:
                         raise ValueError("authors")
                 else:
@@ -618,38 +630,35 @@ class Subscription:
         return select, new_filters
 
 
-class BaseGarbageCollector:
+class BaseGarbageCollector(Periodic):
     def __init__(self, db, **kwargs):
         self.log = logging.getLogger("nostr_relay.db:gc")
         self.db = db
         self.running = True
         self.collect_interval = kwargs.get('collect_interval', 300)
+        super().__init__(self.collect_interval)
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-    def collect(self, db):
+    async def collect(self, db):
         pass
 
     async def start(self):
         self.log.info("Starting garbage collector %s. Interval %s", self.__class__.__name__, self.collect_interval)
-        while self.running:
-            await asyncio.sleep(self.collect_interval)
-            collected = 0
-            try:
-                async with self.db.begin() as conn:
-                    collected = await self.collect(conn)
-                if collected:
-                    self.log.info("Collected garbage (%d events)", collected)
-            except sqlite3.OperationalError as e:
-                self.log.exception("collect")
-                break
-            except Exception:
-                self.log.exception("collect")
-                continue
-        self.log.info("Stopped")
+        await super().start()
 
-    def stop(self):
-        self.running = False
+    async def run_once(self):
+        collected = 0
+        try:
+            async with self.db.begin() as conn:
+                collected = await self.collect(conn)
+            if collected:
+                self.log.info("Collected garbage (%d events)", collected)
+        except sqlite3.OperationalError as e:
+            self.log.exception("collect")
+            self.running = False
+        except Exception:
+            self.log.exception("collect")
 
 
 class QueryGarbageCollector(BaseGarbageCollector):
