@@ -18,6 +18,7 @@ from datetime import datetime, timedelta
 import sqlalchemy as sa
 
 from .errors import VerificationError
+from .util import Periodic
 from nostr_relay.storage import get_metadata
 
 
@@ -39,14 +40,16 @@ sa.Index('verifiedidx', Verification.c.verified_at)
 EventTable = _metadata.tables['events']
 
 
-class Verifier:
+class Verifier(Periodic):
 
     def __init__(self, storage, options:dict = None):
         self.storage = storage
         self.options = options or {}
         self.options.setdefault('update_frequency', 3600)
         self.options.setdefault('expiration', 86400)
-        self.running = True
+        Periodic.__init__(self, self.options['update_frequency'])
+        self._candidate = None
+        self._last_run = 0
         self.queue = asyncio.Queue()
         # nip05_verification can be "enabled", "disabled", or "passive"
         self.is_enabled = options.get('nip05_verification', '') == 'enabled'
@@ -136,61 +139,53 @@ class Verifier:
         self.log.debug("Verified %s", event.pubkey)
         return True
 
-    async def verification_task(self, db):
-        self.log.info("Starting verification task. Interval %s", self.options['update_frequency'])
-        last_run = 0
-        while self.running:
-            candidate = await self.queue.get()
-            if candidate is None:
-                break
-            self.log.debug("Got candidate %s", candidate)
-            candidates = []
-            try:
-                if (time.time() - last_run) > self.options['update_frequency']:
-                    self.log.debug("running batch query")
-                    query = sa.select(Verification.c.id, Verification.c.identifier, Verification.c.verified_at, EventTable.c.pubkey, Verification.c.metadata_id).select_from(
-                        sa.join(Verification, self.storage.EventTable, Verification.c.metadata_id == self.storage.EventTable.c.id, isouter=True)
-                    ).where(
-                        (EventTable.c.pubkey != None) & (Verification.c.verified_at > (datetime.utcnow() - timedelta(seconds=self.options['expiration'])))
-                    )
+    async def run_once(self):
+        candidates = []
+        try:
+            if (time.time() - self._last_run) > self.options['update_frequency']:
+                self.log.debug("running batch query %s", self._last_run)
+                query = sa.select(Verification.c.id, Verification.c.identifier, Verification.c.verified_at, EventTable.c.pubkey, Verification.c.metadata_id).select_from(
+                    sa.join(Verification, self.storage.EventTable, Verification.c.metadata_id == self.storage.EventTable.c.id, isouter=True)
+                ).where(
+                    (EventTable.c.pubkey != None) & (Verification.c.verified_at > (datetime.utcnow() - timedelta(seconds=self.options['expiration'])))
+                )
 
-                    async with db.begin() as cursor:
-                        try:
-                            results = await cursor.stream(query)
-                            async for row in results:
-                                candidates.append(row)
-                        except Exception:
-                            self.log.exception('batch query')
+                async with self.db.begin() as cursor:
+                    try:
+                        results = await cursor.stream(query)
+                        async for row in results:
+                            candidates.append(row)
+                    except Exception:
+                        self.log.exception('batch query')
+                        return
+                        # vid, identifier, verified_at, pubkey = row
+        except Exception:
+            self.log.exception("batch_query")
+            return
+        if self._candidate:
+            candidates.append(self._candidate)
+
+        try:
+            success, failure = await self.process_verifications(candidates)
+        except Exception:
+            self.log.exception("process_verifications")
+        else:
+            if success or failure:
+                async with self.db.begin() as conn:
+                    for vid, identifier, metadata_id in success:
+                        if vid is None:
+                            # first time verifying
+                            await conn.execute(sa.insert(Verification).values({'identifier': identifier, 'metadata_id': metadata_id, 'verified_at': datetime.utcnow()}))
+                        else:
+                            await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'verified_at': datetime.utcnow()}))
+                    for vid, identifier, metadata_id in failure:
+                        if vid is None:
+                            # don't persist first time candidates
                             continue
-                            # vid, identifier, verified_at, pubkey = row
-            except Exception:
-                self.log.exception("batch_query")
-                continue
-            candidates.append(candidate)
-
-            try:
-                success, failure = await self.process_verifications(candidates)
-            except Exception:
-                self.log.exception("process_verifications")
-            else:
-                if success or failure:
-                    async with db.begin() as conn:
-                        for vid, identifier, metadata_id in success:
-                            if vid is None:
-                                # first time verifying
-                                await conn.execute(sa.insert(Verification).values({'identifier': identifier, 'metadata_id': metadata_id, 'verified_at': datetime.utcnow()}))
-                            else:
-                                await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'verified_at': datetime.utcnow()}))
-                        for vid, identifier, metadata_id in failure:
-                            if vid is None:
-                                # don't persist first time candidates
-                                continue
-                            else:
-                                await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'failed_at': datetime.utcnow()}))
-                    self.log.info("Saved success:%d failure:%d", len(success), len(failure))
-            last_run = time.time()
-
-        self.log.info("Stopped verification task")
+                        else:
+                            await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'failed_at': datetime.utcnow()}))
+                self.log.info("Saved success:%d failure:%d", len(success), len(failure))
+        self._last_run = time.time()
 
     def get_aiohttp_session(self):
         import aiohttp
@@ -232,14 +227,23 @@ class Verifier:
 
         return success, failure
 
+    async def wait_function(self):
+        self._candidate = None
+        try:
+            async with asyncio.timeout(self.options['update_frequency']):
+                self._candidate = await self.queue.get()
+            if self._candidate is None:
+                self.running = False
+            self.log.debug("Got candidate %s", self._candidate)
+        except asyncio.TimeoutError:
+            self.log.debug("timed out waiting for queue")
+
     async def start(self, db):
-        if self.should_verify:
-            asyncio.create_task(self.verification_task(db))
 
-    async def stop(self):
         if self.should_verify:
-            self.running = False
-            await self.queue.put(None)
+            self.log.info("Starting verification task. Interval %s", self.options['update_frequency'])
+            self.db = db
+            await super().start()
 
     def is_processing(self):
         return not self.queue.empty()
