@@ -145,7 +145,7 @@ class DBTests(BaseTestsWithStorage):
         assert changed
         import json
         async for event in self.storage.run_single_query([{"authors": ["86f0689bd48dcd19c67a19d994f938ee34f251d8c39976290955ff585f2db42e"]}]):
-            assert json.loads(event)['id'] == DELEGATION_EVENT['id']
+            assert event.id == DELEGATION_EVENT['id']
 
     async def test_get_stats(self):
         for evt in EVENTS:
@@ -408,6 +408,36 @@ class MainTests(APITests):
             data = await self.send_event(ws, EVENTS[1], True)
             assert data[3] == 'rate-limited: slow down'
 
+    async def test_timeout(self):
+        """
+        Test time out waiting for next message
+        """
+        Config.message_timeout = 1.5
+        async with self.conductor.simulate_ws("/") as ws:
+            await self.send_event(ws, EVENTS[1], True)
+            await asyncio.sleep(1.6)
+            with self.assertRaises(errors.WebSocketDisconnected):
+                data = await ws.send_json(["REQ", "test", {"kinds": 0}])
+
+        Config.message_timeout = None
+
+    async def test_too_many_subscriptions(self):
+        """
+        Test subscription limit
+        """
+        Config.subscription_limit = 2
+        async with self.conductor.simulate_ws("/") as ws:
+            await ws.send_json(["REQ", "t1", {"kinds": [1,2]}])
+            data = await ws.receive_json()
+            assert data == ['EOSE', 't1']
+            await ws.send_json(["REQ", "t2", {"kinds": [3,4]}])
+            data = await ws.receive_json()
+            assert data == ['EOSE', 't2']
+            await ws.send_json(["REQ", "t3", {"kinds": [5,6]}])
+            data = await ws.receive_json()
+            assert data == ['NOTICE', 'rejected: too many subscriptions']
+        Config.subscription_limit = None
+
 
 class AuthTests(APITests):
     async def asyncSetUp(self):
